import time

import requests
from requests.exceptions import RequestException

from akshare.exceptions import NetworkError, APIError, RateLimitError, DataParsingError
from akshare.utils.context import config


def make_request_with_retry_json(
    url, params=None, headers=None, proxies=None, max_retries=3, retry_delay=1
):
    """
    发送 HTTP GET 请求，支持重试机制和代理设置。

    :param url: 请求的 URL
    :param params: URL 参数 (可选)
    :param headers: 请求头 (可选)
    :param proxies: 代理设置 (可选)
    :param max_retries: 最大重试次数
    :param retry_delay: 初始重试延迟（秒）
    :return: 解析后的 JSON 数据
    """
    if proxies is None:
        proxies = config.proxies
    for attempt in range(max_retries):
        try:
            response = requests.get(
                url, params=params, headers=headers, proxies=proxies
            )
            if response.status_code == 200:
                try:
                    data = response.json()
                    if not data:
                        raise DataParsingError("Empty response data")
                    return data
                except ValueError:
                    raise DataParsingError("Failed to parse JSON response")
            elif response.status_code == 429:
                raise RateLimitError(
                    f"Rate limit exceeded. Status code: {response.status_code}"
                )
            else:
                raise APIError(
                    f"API request failed. Status code: {response.status_code}"
                )

        except (RequestException, RateLimitError, APIError, DataParsingError) as e:
            if attempt == max_retries - 1:
                if isinstance(e, RateLimitError):
                    raise
                elif isinstance(e, (APIError, DataParsingError)):
                    raise
                else:
                    raise NetworkError(
                        f"Failed to connect after {max_retries} attempts: {str(e)}"
                    )

            time.sleep(retry_delay)
            retry_delay *= 2  # 指数退避策略

    raise NetworkError(f"Failed to connect after {max_retries} attempts")


def make_request_with_retry_text(
    url, params=None, headers=None, proxies=None, max_retries=3, retry_delay=1
):
    """
    发送 HTTP GET 请求，支持重试机制和代理设置。

    :param url: 请求的 URL
    :param params: URL 参数 (可选)
    :param headers: 请求头 (可选)
    :param proxies: 代理设置 (可选)
    :param max_retries: 最大重试次数
    :param retry_delay: 初始重试延迟（秒）
    :return: 解析后的 JSON 数据
    """
    if proxies is None:
        proxies = config.proxies
    for attempt in range(max_retries):
        try:
            response = requests.get(
                url, params=params, headers=headers, proxies=proxies
            )
            if response.status_code == 200:
                try:
                    data = response.text
                    if not data:
                        raise DataParsingError("Empty response data")
                    return data
                except ValueError:
                    raise DataParsingError("Failed to parse JSON response")
            elif response.status_code == 429:
                raise RateLimitError(
                    f"Rate limit exceeded. Status code: {response.status_code}"
                )
            else:
                raise APIError(
                    f"API request failed. Status code: {response.status_code}"
                )

        except (RequestException, RateLimitError, APIError, DataParsingError) as e:
            if attempt == max_retries - 1:
                if isinstance(e, RateLimitError):
                    raise
                elif isinstance(e, (APIError, DataParsingError)):
                    raise
                else:
                    raise NetworkError(
                        f"Failed to connect after {max_retries} attempts: {str(e)}"
                    )

            time.sleep(retry_delay)
            retry_delay *= 2  # 指数退避策略

    raise NetworkError(f"Failed to connect after {max_retries} attempts")



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/5/11 16:00
Desc: 东方财富-港股-公司概况
https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/CompanyProfile
"""

import pandas as pd
import requests


def stock_hk_security_profile_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-证券资料
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/CompanyProfile
    :param symbol: 股票代码
    :type symbol: str
    :return: 证券资料
    :rtype: pandas.DataFrame
    """
    url = 'https://datacenter.eastmoney.com/securities/api/data/v1/get'
    params = {
        'reportName': 'RPT_HKF10_INFO_SECURITYINFO',
        'columns': 'SECUCODE,SECURITY_CODE,SECURITY_NAME_ABBR,SECURITY_TYPE,LISTING_DATE,ISIN_CODE,BOARD,'
                   'TRADE_UNIT,TRADE_MARKET,GANGGUTONGBIAODISHEN,GANGGUTONGBIAODIHU,PAR_VALUE,'
                   'ISSUE_PRICE,ISSUE_NUM,YEAR_SETTLE_DAY',
        'quoteColumns': '',
        'filter': f'(SECUCODE="{symbol}.HK")',
        'pageNumber': '1',
        'pageSize': '200',
        'sortTypes': '',
        'sortColumns': '',
        'source': 'F10',
        'client': 'PC',
        'v': '04748497219912483'
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['result']['data'])
    field_mapping = {
        "BOARD": "板块",
        "GANGGUTONGBIAODIHU": "是否沪港通标的",
        "GANGGUTONGBIAODISHEN": "是否深港通标的",
        "ISIN_CODE": "ISIN（国际证券识别编码）",
        "ISSUE_NUM": "发行量(股)",
        "ISSUE_PRICE": "发行价",
        "LISTING_DATE": "上市日期",
        "PAR_VALUE": "每股面值",
        "SECUCODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "SECURITY_TYPE": "证券类型",
        "TRADE_MARKET": "交易所",
        "TRADE_UNIT": "每手股数",
        "YEAR_SETTLE_DAY": "年结日"
    }
    temp_df.rename(columns=field_mapping, inplace=True)
    temp_df = temp_df[[
        "证券代码",
        "证券简称",
        "上市日期",
        "证券类型",
        "发行价",
        "发行量(股)",
        "每手股数",
        "每股面值",
        "交易所",
        "板块",
        "年结日",
        "ISIN（国际证券识别编码）",
        "是否沪港通标的",
        "是否深港通标的",
    ]]
    return temp_df


def stock_hk_company_profile_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-公司资料
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/CompanyProfile
    :param symbol: 股票代码
    :type symbol: str
    :return: 公司资料
    :rtype: pandas.DataFrame
    """
    url = 'https://datacenter.eastmoney.com/securities/api/data/v1/get'
    params = {
        'reportName': 'RPT_HKF10_INFO_ORGPROFILE',
        'columns': 'SECUCODE,SECURITY_CODE,ORG_NAME,ORG_EN_ABBR,BELONG_INDUSTRY,FOUND_DATE,CHAIRMAN,'
                   'SECRETARY,ACCOUNT_FIRM,REG_ADDRESS,ADDRESS,YEAR_SETTLE_DAY,EMP_NUM,ORG_TEL,ORG_FAX,ORG_EMAIL,'
                   'ORG_WEB,ORG_PROFILE,REG_PLACE',
        'quoteColumns': '',
        'filter': f'(SECUCODE="{symbol}.HK")',
        'pageNumber': '1',
        'pageSize': '200',
        'sortTypes': '',
        'sortColumns': '',
        'source': 'F10',
        'client': 'PC',
        'v': '04748497219912483'
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['result']['data'])
    field_mapping = {
        'ACCOUNT_FIRM': '核数师',
        'ADDRESS': '办公地址',
        'BELONG_INDUSTRY': '所属行业',
        'CHAIRMAN': '董事长',
        'EMP_NUM': '员工人数',
        'FOUND_DATE': '公司成立日期',
        'ORG_EMAIL': 'E-MAIL',
        'ORG_EN_ABBR': '英文名称',
        'ORG_FAX': '传真',
        'ORG_NAME': '公司名称',
        'ORG_PROFILE': '公司介绍',
        'ORG_TEL': '联系电话',
        'ORG_WEB': '公司网址',
        'REG_ADDRESS': '注册地址',
        'REG_PLACE': '注册地',
        'SECRETARY': '公司秘书',
        'SECUCODE': '股票代码',
        'SECURITY_CODE': '证券代码',
        'YEAR_SETTLE_DAY': '年结日'
    }
    temp_df.rename(columns=field_mapping, inplace=True)
    temp_df = temp_df[[
        "公司名称",
        "英文名称",
        "注册地",
        "注册地址",
        "公司成立日期",
        "所属行业",
        "董事长",
        "公司秘书",
        "员工人数",
        "办公地址",
        "公司网址",
        "E-MAIL",
        "年结日",
        "联系电话",
        "核数师",
        "传真",
        "公司介绍",
    ]]
    return temp_df


if __name__ == "__main__":
    stock_hk_security_profile_em_df = stock_hk_security_profile_em(symbol="03900")
    print(stock_hk_security_profile_em_df)

    stock_hk_company_profile_em_df = stock_hk_company_profile_em(symbol="03900")
    print(stock_hk_company_profile_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/5 20:20
Desc: 生意社-商品与期货-现期图
https://www.100ppi.com/sf/792.html
"""

from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup


def __get_sys_spot_futures_dict() -> dict:
    """
    生意社-商品与期货-现期图: 品种和网址字典
    https://www.100ppi.com/sf/792.html
    :return: 品种和网址字典
    :rtype: dict
    """
    url = "https://www.100ppi.com/sf/792.html"
    res = requests.get(url)
    soup = BeautifulSoup(res.text, features="lxml")
    temp_item = soup.find(name="div", attrs={"class": "q8"}).find_all("li")
    name_url_dict = dict(
        zip(
            [item.find("a").get_text().strip() for item in temp_item],
            [item.find("a")["href"] for item in temp_item],
        )
    )
    return name_url_dict


def futures_spot_sys(symbol: str = "铜", indicator: str = "市场价格") -> pd.DataFrame:
    """
    生意社-商品与期货-现期图
    https://www.100ppi.com/sf/792.html
    :param symbol: 期货品种
    :type symbol: str
    :param indicator: 市场价格; choice of {"市场价格", "基差率", "主力基差"}
    :type indicator: str
    :return: pandas.DataFrame
    :rtype: dict
    """
    name_url_dict = __get_sys_spot_futures_dict()
    url = name_url_dict[symbol]
    r = requests.get("https://www.100ppi.com" + url)
    if indicator == "市场价格":
        table_df_one = pd.read_html(StringIO(r.text), header=0, index_col=0)[1].T
        table_df_one["现货价格"] = pd.to_numeric(
            table_df_one["现货价格"], errors="coerce"
        )
        table_df_one["主力合约"] = pd.to_numeric(
            table_df_one["主力合约"], errors="coerce"
        )
        table_df_one["最近合约"] = pd.to_numeric(
            table_df_one["最近合约"], errors="coerce"
        )
        table_df_one.reset_index(inplace=True)
        table_df_one.columns.name = None
        table_df_one.rename(columns={"index": "日期"}, inplace=True)
        return table_df_one
    elif indicator == "基差率":
        table_df_two = pd.read_html(StringIO(r.text), header=0, index_col=0)[2].T
        table_df_two["基差率"] = table_df_two["基差率"].str.replace("%", "")
        table_df_two["基差率"] = pd.to_numeric(table_df_two["基差率"], errors="coerce")
        table_df_two.reset_index(inplace=True)
        table_df_two.columns.name = None
        table_df_two.rename(columns={"index": "日期"}, inplace=True)
        return table_df_two
    else:
        table_df_three = pd.read_html(StringIO(r.text), header=0, index_col=0)[3].T
        table_df_three["主力基差"] = pd.to_numeric(
            table_df_three["主力基差"], errors="coerce"
        )
        table_df_three.reset_index(inplace=True)
        table_df_three.columns.name = None
        table_df_three.rename(columns={"index": "日期"}, inplace=True)
        return table_df_three


if __name__ == "__main__":
    futures_spot_sys_df = futures_spot_sys(symbol="铜", indicator="市场价格")
    print(futures_spot_sys_df)

    futures_spot_sys_df = futures_spot_sys(symbol="铜", indicator="基差率")
    print(futures_spot_sys_df)

    futures_spot_sys_df = futures_spot_sys(symbol="铜", indicator="主力基差")
    print(futures_spot_sys_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2025/3/7 20:30
Desc: 百度股市通-经济数据
https://gushitong.baidu.com/calendar
"""

import pandas as pd
import requests


def news_economic_baidu(date: str = "20241107") -> pd.DataFrame:
    """
    百度股市通-经济数据
    https://gushitong.baidu.com/calendar
    :param date: 查询日期
    :type date: str
    :return: 经济数据
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([date[:4], date[4:6], date[6:]])
    end_date = "-".join([date[:4], date[4:6], date[6:]])
    url = "https://finance.pae.baidu.com/api/financecalendar"
    params = {
        "start_date": start_date,
        "end_date": end_date,
        "market": "",
        "cate": "economic_data",
        "finClientType": "pc",
    }
    r = requests.get(url=url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    for item in data_json["Result"]:
        if not item["list"] == []:
            temp_df = pd.DataFrame(item["list"])
            temp_df.columns = [
                "日期",
                "时间",
                "-",
                "事件",
                "重要性",
                "前值",
                "预期",
                "公布",
                "-",
                "-",
                "地区",
                "-",
            ]
            temp_df = temp_df[
                [
                    "日期",
                    "时间",
                    "地区",
                    "事件",
                    "公布",
                    "预期",
                    "前值",
                    "重要性",
                ]
            ]
            temp_df["公布"] = pd.to_numeric(temp_df["公布"], errors="coerce")
            temp_df["预期"] = pd.to_numeric(temp_df["预期"], errors="coerce")
            temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
            temp_df["重要性"] = pd.to_numeric(temp_df["重要性"], errors="coerce")
            temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
            big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
        else:
            continue
    return big_df


def news_trade_notify_suspend_baidu(date: str = "20241107") -> pd.DataFrame:
    """
    百度股市通-交易提醒-停复牌
    https://gushitong.baidu.com/calendar
    :param date: 查询日期
    :type date: str
    :return: 交易提醒-停复牌
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([date[:4], date[4:6], date[6:]])
    end_date = "-".join([date[:4], date[4:6], date[6:]])
    url = "https://finance.pae.baidu.com/api/financecalendar"
    params = {
        "start_date": start_date,
        "end_date": end_date,
        "market": "",
        "cate": "notify_suspend",
        "finClientType": "pc",
    }
    r = requests.get(url=url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    for item in data_json["Result"]:
        if not item["list"] == []:
            temp_df = pd.DataFrame(item["list"])
            temp_df.columns = [
                "股票代码",
                "-",
                "交易所",
                "股票简称",
                "停牌时间",
                "复牌时间",
                "-",
                "停牌事项说明",
            ]
            temp_df = temp_df[
                [
                    "股票代码",
                    "股票简称",
                    "交易所",
                    "停牌时间",
                    "复牌时间",
                    "停牌事项说明",
                ]
            ]
            temp_df["停牌时间"] = pd.to_datetime(
                temp_df["停牌时间"], errors="coerce"
            ).dt.date
            temp_df["复牌时间"] = pd.to_datetime(
                temp_df["复牌时间"], errors="coerce"
            ).dt.date
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
        else:
            continue
    return big_df


def news_trade_notify_dividend_baidu(date: str = "20241107") -> pd.DataFrame:
    """
    百度股市通-交易提醒-分红派息
    https://gushitong.baidu.com/calendar
    :param date: 查询日期
    :type date: str
    :return: 交易提醒-分红派息
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([date[:4], date[4:6], date[6:]])
    end_date = "-".join([date[:4], date[4:6], date[6:]])
    url = "https://finance.pae.baidu.com/api/financecalendar"
    params = {
        "start_date": start_date,
        "end_date": end_date,
        "market": "",
        "cate": "notify_divide",
        "rn": 500,
        "pn": 0,
    }
    r = requests.get(url=url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    for item in data_json["Result"]:
        if not item["list"] == []:
            temp_df = pd.DataFrame(item["list"])
            temp_df.rename(
                columns={
                    "code": "股票代码",
                    "market": "-",
                    "exchange": "交易所",
                    "name": "股票简称",
                    "diviDate": "除权日",
                    "date": "报告期",
                    "diviCash": "分红",
                    "shareDivide": "送股",
                    "transfer": "转增",
                    "physical": "实物",
                },
                inplace=True,
            )
            if "实物" not in temp_df.columns:
                temp_df["实物"] = pd.NA
            temp_df = temp_df[
                [
                    "股票代码",
                    "除权日",
                    "分红",
                    "送股",
                    "转增",
                    "实物",
                    "交易所",
                    "股票简称",
                    "报告期",
                ]
            ]
            temp_df["除权日"] = pd.to_datetime(
                temp_df["除权日"], errors="coerce"
            ).dt.date
            temp_df["报告期"] = pd.to_datetime(
                temp_df["报告期"], errors="coerce"
            ).dt.date
            big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
        else:
            continue
    return big_df


def news_report_time_baidu(date: str = "20241107") -> pd.DataFrame:
    """
    百度股市通-财报发行
    https://gushitong.baidu.com/calendar
    :param date: 查询日期
    :type date: str
    :return: 财报发行
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([date[:4], date[4:6], date[6:]])
    end_date = "-".join([date[:4], date[4:6], date[6:]])
    url = "https://finance.pae.baidu.com/api/financecalendar"
    params = {
        "start_date": start_date,
        "end_date": end_date,
        "market": "",
        "cate": "report_time",
        "finClientType": "pc",
    }
    r = requests.get(url=url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    for item in data_json["Result"]:
        if not item["list"] == []:
            temp_df = pd.DataFrame(item["list"])
            temp_df.columns = [
                "股票代码",
                "-",
                "交易所",
                "-",
                "股票简称",
                "-",
                "财报期",
            ]
            temp_df = temp_df[
                [
                    "股票代码",
                    "交易所",
                    "股票简称",
                    "财报期",
                ]
            ]
            big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
        else:
            continue
    return big_df


if __name__ == "__main__":
    news_economic_baidu_df = news_economic_baidu(date="20250307")
    print(news_economic_baidu_df)

    news_trade_notify_suspend_baidu_df = news_trade_notify_suspend_baidu(
        date="20241107"
    )
    print(news_trade_notify_suspend_baidu_df)

    news_trade_notify_dividend_baidu_df = news_trade_notify_dividend_baidu(
        date="20250506"
    )
    print(news_trade_notify_dividend_baidu_df)

    news_report_time_baidu_df = news_report_time_baidu(date="20241107")
    print(news_report_time_baidu_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2025/3/11 17:00
Desc: 东方财富网-数据中心-特色数据-期权价值分析
https://data.eastmoney.com/other/valueAnal.html
"""

import pandas as pd

from akshare.utils.func import fetch_paginated_data


def option_value_analysis_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-期权价值分析
    https://data.eastmoney.com/other/valueAnal.html
    :return: 期权价值分析
    :rtype: pandas.DataFrame
    """
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        "fid": "f301",
        "po": "1",
        "pz": "100",
        "pn": "1",
        "np": "1",
        "fltt": "2",
        "invt": "2",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "fields": "f1,f2,f3,f12,f13,f14,f298,f299,f249,f300,f330,f331,f332,f333,f334,f335,f336,f301,f152",
        "fs": "m:10",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.columns = [
        "-",
        "-",
        "最新价",
        "-",
        "期权代码",
        "-",
        "期权名称",
        "-",
        "隐含波动率",
        "时间价值",
        "内在价值",
        "理论价格",
        "到期日",
        "-",
        "-",
        "-",
        "标的名称",
        "标的最新价",
        "-",
        "标的近一年波动率",
    ]
    temp_df = temp_df[
        [
            "期权代码",
            "期权名称",
            "最新价",
            "时间价值",
            "内在价值",
            "隐含波动率",
            "理论价格",
            "标的名称",
            "标的最新价",
            "标的近一年波动率",
            "到期日",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["时间价值"] = pd.to_numeric(temp_df["时间价值"], errors="coerce")
    temp_df["内在价值"] = pd.to_numeric(temp_df["内在价值"], errors="coerce")
    temp_df["隐含波动率"] = pd.to_numeric(temp_df["隐含波动率"], errors="coerce")
    temp_df["理论价格"] = pd.to_numeric(temp_df["理论价格"], errors="coerce")
    temp_df["标的最新价"] = pd.to_numeric(temp_df["标的最新价"], errors="coerce")
    temp_df["标的近一年波动率"] = pd.to_numeric(
        temp_df["标的近一年波动率"], errors="coerce"
    )
    temp_df["到期日"] = pd.to_datetime(
        temp_df["到期日"].astype(str), errors="coerce"
    ).dt.date
    return temp_df


if __name__ == "__main__":
    option_value_analysis_em_df = option_value_analysis_em()
    print(option_value_analysis_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/2/28 11:00
Desc: 大连商品交易所-业务/服务-业务参数-交易参数-合约信息查询
http://www.dce.com.cn/dalianshangpin/ywfw/ywcs/jycs/hyxxcx/index.html
"""

from io import StringIO

import pandas as pd
import requests


def futures_contract_info_dce() -> pd.DataFrame:
    """
    大连商品交易所-业务/服务-业务参数-交易参数-合约信息查询
    http://www.dce.com.cn/dalianshangpin/ywfw/ywcs/jycs/hyxxcx/index.html
    :return: 交易参数汇总查询
    :rtype: pandas.DataFrame
    """
    url = "http://www.dce.com.cn/publicweb/businessguidelines/queryContractInfo.html"
    params = {
        "contractInformation.variety": "all",
        "contractInformation.trade_type": "0",
    }
    r = requests.post(url, params=params)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df["交易单位"] = pd.to_numeric(temp_df["交易单位"], errors="coerce")
    temp_df["最小变动价位"] = pd.to_numeric(temp_df["最小变动价位"], errors="coerce")
    temp_df["最小变动价位"] = pd.to_numeric(temp_df["最小变动价位"], errors="coerce")
    temp_df["开始交易日"] = pd.to_datetime(
        temp_df["开始交易日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["最后交易日"] = pd.to_datetime(
        temp_df["最后交易日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["最后交割日"] = pd.to_datetime(
        temp_df["最后交割日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    return temp_df


if __name__ == "__main__":
    futures_contract_info_dce_df = futures_contract_info_dce()
    print(futures_contract_info_dce_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/22 20:30
Desc: 期货-期转现-交割
"""

from io import StringIO, BytesIO

import pandas as pd
import requests


def futures_to_spot_shfe(date: str = "202312") -> pd.DataFrame:
    """
    上海期货交易所-期转现
    https://tsite.shfe.com.cn/statements/dataview.html?paramid=kx
    1、铜、铜(BC)、铝、锌、铅、镍、锡、螺纹钢、线材、热轧卷板、天然橡胶、20号胶、低硫燃料油、燃料油、石油沥青、纸浆、不锈钢的数量单位为：吨；黄金的数量单位为：克；白银的数量单位为：千克；原油的数量单位为：桶。
    2、交割量、期转现量为单向计算。
    :param date: 年月
    :type date: str
    :return: 上海期货交易所期转现
    :rtype: pandas.DataFrame
    """
    url = f"https://tsite.shfe.com.cn/data/instrument/ExchangeDelivery{date}.dat"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/100.0.4896.127 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["ExchangeDelivery"])
    temp_df.columns = [
        "_",
        "日期",
        "交割量",
        "_",
        "期转现量",
        "合约",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "日期",
            "合约",
            "交割量",
            "期转现量",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["交割量"] = pd.to_numeric(temp_df["交割量"], errors="coerce")
    temp_df["期转现量"] = pd.to_numeric(temp_df["期转现量"], errors="coerce")
    return temp_df


def futures_delivery_dce(date: str = "202312") -> pd.DataFrame:
    """
    大连商品交易所-交割统计
    http://www.dce.com.cn/dalianshangpin/xqsj/tjsj26/jgtj/jgsj/index.html
    :param date: 交割日期
    :type date: str
    :return: 大连商品交易所-交割统计
    :rtype: pandas.DataFrame
    """
    url = "http://www.dce.com.cn/publicweb/quotesdata/delivery.html"
    params = {
        "deliveryQuotes.variety": "all",
        "year": "",
        "month": "",
        "deliveryQuotes.begin_month": date,
        "deliveryQuotes.end_month": str(int(date) + 1),
    }
    headers = {
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Pragma": "no-cache",
        "Upgrade-Insecure-Requests": "1",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.127 Safari/537.36",
    }
    r = requests.post(url, params=params, headers=headers)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df["交割日期"] = (
        temp_df["交割日期"].astype(str).str.split(".", expand=True).iloc[:, 0]
    )
    temp_df = temp_df[~temp_df["品种"].str.contains("小计|总计")]
    temp_df.reset_index(inplace=True, drop=True)
    temp_df["交割日期"] = pd.to_datetime(temp_df["交割日期"], errors="coerce").dt.date
    temp_df["交割量"] = pd.to_numeric(temp_df["交割量"], errors="coerce")
    temp_df["交割金额"] = pd.to_numeric(temp_df["交割金额"], errors="coerce")
    return temp_df


def futures_to_spot_dce(date: str = "202312") -> pd.DataFrame:
    """
    大连商品交易所-期转现
    http://www.dce.com.cn/dalianshangpin/xqsj/tjsj26/jgtj/qzxcx/index.html
    :param date: 期转现日期
    :type date: str
    :return: 大连商品交易所-期转现
    :rtype: pandas.DataFrame
    """
    url = "http://www.dce.com.cn/publicweb/quotesdata/ftsDeal.html"
    params = {
        "ftsDealQuotes.variety": "all",
        "year": "",
        "month": "",
        "ftsDealQuotes.begin_month": date,
        "ftsDealQuotes.end_month": date,
    }
    r = requests.post(url, params=params)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df["期转现发生日期"] = (
        temp_df["期转现发生日期"].astype(str).str.split(".", expand=True).iloc[:, 0]
    )
    temp_df = temp_df[~temp_df["合约代码"].str.contains("小计|总计")]
    temp_df.reset_index(inplace=True, drop=True)
    temp_df["期转现发生日期"] = pd.to_datetime(
        temp_df["期转现发生日期"], errors="coerce"
    ).dt.date
    temp_df["期转现数量"] = pd.to_numeric(temp_df["期转现数量"], errors="coerce")
    return temp_df


def futures_delivery_match_dce(symbol: str = "a") -> pd.DataFrame:
    """
    大连商品交易所-交割配对表
    http://www.dce.com.cn/dalianshangpin/xqsj/tjsj26/jgtj/jgsj/index.html
    :param symbol: 交割品种
    :type symbol: str
    :return: 大连商品交易所-交割配对表
    :rtype: pandas.DataFrame
    """
    url = "http://www.dce.com.cn/publicweb/quotesdata/deliveryMatch.html"
    params = {
        "deliveryMatchQuotes.variety": symbol,
        "contract.contract_id": "all",
        "contract.variety_id": symbol,
    }
    r = requests.post(url, params=params)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df["配对日期"] = (
        temp_df["配对日期"].astype(str).str.split(".", expand=True).iloc[:, 0]
    )
    temp_df = temp_df.iloc[:-1, :]
    temp_df["配对日期"] = pd.to_datetime(temp_df["配对日期"], errors="coerce").dt.date
    temp_df["配对手数"] = pd.to_numeric(temp_df["配对手数"], errors="coerce")
    temp_df["交割结算价"] = pd.to_numeric(temp_df["交割结算价"], errors="coerce")
    return temp_df


def futures_to_spot_czce(date: str = "20231228") -> pd.DataFrame:
    """
    郑州商品交易所-期转现统计
    http://www.czce.com.cn/cn/jysj/qzxtj/H770311index_1.htm
    :param date: 年月日
    :type date: str
    :return: 郑州商品交易所-期转现统计
    :rtype: pandas.DataFrame
    """
    url = f"http://www.czce.com.cn/cn/DFSStaticFiles/Future/{date[:4]}/{date}/FutureDataTrdtrades.xls"
    headers = {
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "www.czce.com.cn",
        "Pragma": "no-cache",
        "Referer": "http://www.czce.com.cn/",
        "Upgrade-Insecure-Requests": "1",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.127 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    r.encoding = "utf-8"
    temp_df = pd.read_excel(BytesIO(r.content), skiprows=1)

    temp_df.columns = [
        "合约代码",
        "合约数量",
    ]
    temp_df = temp_df[
        [
            "合约代码",
            "合约数量",
        ]
    ]
    temp_df["合约数量"] = temp_df["合约数量"].astype(str).str.replace(",", "")
    temp_df["合约数量"] = pd.to_numeric(temp_df["合约数量"], errors="coerce")
    temp_df = temp_df[~temp_df["合约代码"].str.contains("小计|合计")]
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


def futures_delivery_match_czce(date: str = "20210106") -> pd.DataFrame:
    """
    郑州商品交易所-交割配对
    http://www.czce.com.cn/cn/jysj/jgpd/H770308index_1.htm
    :param date: 年月日
    :type date: str
    :return: 郑州商品交易所-交割配对
    :rtype: pandas.DataFrame
    """
    url = f"http://www.czce.com.cn/cn/DFSStaticFiles/Future/{date[:4]}/{date}/FutureDataDelsettle.xls"
    r = requests.get(url)
    r.encoding = "utf-8"
    temp_df = pd.read_excel(BytesIO(r.content), skiprows=0)
    index_flag = temp_df[temp_df.iloc[:, 0].str.contains("配对日期")].index.values
    big_df = pd.DataFrame()
    for i, item in enumerate(index_flag):
        try:
            temp_inner_df = temp_df[index_flag[i] + 1 : index_flag[i + 1]]
        except:  # noqa: E722
            temp_inner_df = temp_df[index_flag[i] + 1 :]
        temp_inner_df.columns = temp_inner_df.iloc[0, :]
        temp_inner_df = temp_inner_df.iloc[1:-1, :]
        temp_inner_df.reset_index(drop=True, inplace=True)
        date_contract_str = (
            temp_df[temp_df.iloc[:, 0].str.contains("配对日期")].iloc[:, 0].values[i]
        )
        inner_date = date_contract_str.split("：")[1].split(" ")[0]
        symbol = date_contract_str.split("：")[-1]
        temp_inner_df["配对日期"] = inner_date
        temp_inner_df["合约代码"] = symbol
        big_df = pd.concat([big_df, temp_inner_df], ignore_index=True)

    big_df.columns = [
        "卖方会员",
        "卖方会员-会员简称",
        "买方会员",
        "买方会员-会员简称",
        "交割量",
        "配对日期",
        "合约代码",
    ]
    big_df["交割量"] = big_df["交割量"].str.replace(",", "")
    big_df["交割量"] = pd.to_numeric(big_df["交割量"])
    return big_df


def futures_delivery_czce(date: str = "20210112") -> pd.DataFrame:
    """
    郑州商品交易所-月度交割查询
    http://www.czce.com.cn/cn/jysj/ydjgcx/H770316index_1.htm
    :param date: 年月日
    :type date: str
    :return: 郑州商品交易所-月度交割查询
    :rtype: pandas.DataFrame
    """
    url = f"http://www.czce.com.cn/cn/DFSStaticFiles/Future/{date[:4]}/{date}/FutureDataSettlematched.xls"
    r = requests.get(url)
    r.encoding = "utf-8"
    temp_df = pd.read_excel(BytesIO(r.content), skiprows=1)
    temp_df.columns = [
        "品种",
        "交割数量",
        "交割额",
    ]
    temp_df["交割数量"] = temp_df["交割数量"].str.replace(",", "")
    temp_df["交割额"] = temp_df["交割额"].str.replace(",", "")

    temp_df["交割数量"] = pd.to_numeric(temp_df["交割数量"])
    temp_df["交割额"] = pd.to_numeric(temp_df["交割额"])
    return temp_df


def futures_delivery_shfe(date: str = "202312") -> pd.DataFrame:
    """
    上海期货交易所-交割情况表
    https://tsite.shfe.com.cn/statements/dataview.html?paramid=kx
    注意: 日期 -> 月度统计 -> 下拉到交割情况表
    :param date: 年月日
    :type date: str
    :return: 上海期货交易所-交割情况表
    :rtype: pandas.DataFrame
    """
    url = f"https://tsite.shfe.com.cn/data/dailydata/{date}monthvarietystatistics.dat"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/100.0.4896.127 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    r.encoding = "utf-8"
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["o_curdelivery"])
    temp_df.columns = [
        "品种",
        "品种代码",
        "_",
        "交割量-本月",
        "交割量-比重",
        "交割量-本年累计",
        "交割量-累计同比",
    ]
    temp_df = temp_df[
        [
            "品种",
            "交割量-本月",
            "交割量-比重",
            "交割量-本年累计",
            "交割量-累计同比",
        ]
    ]
    temp_df["交割量-本月"] = pd.to_numeric(temp_df["交割量-本月"], errors="coerce")
    temp_df["交割量-比重"] = pd.to_numeric(temp_df["交割量-比重"], errors="coerce")
    temp_df["交割量-本年累计"] = pd.to_numeric(
        temp_df["交割量-本年累计"], errors="coerce"
    )
    temp_df["交割量-累计同比"] = pd.to_numeric(
        temp_df["交割量-累计同比"], errors="coerce"
    )
    return temp_df


if __name__ == "__main__":
    futures_to_spot_dce_df = futures_to_spot_dce(date="202312")
    print(futures_to_spot_dce_df)

    futures_to_spot_shfe_df = futures_to_spot_shfe(date="202312")
    print(futures_to_spot_shfe_df)

    futures_to_spot_czce_df = futures_to_spot_czce(date="20210112")
    print(futures_to_spot_czce_df)

    futures_delivery_dce_df = futures_delivery_dce(date="202101")
    print(futures_delivery_dce_df)

    futures_delivery_monthly_czce_df = futures_delivery_czce(date="20210112")
    print(futures_delivery_monthly_czce_df)

    futures_delivery_shfe_df = futures_delivery_shfe(date="202312")
    print(futures_delivery_shfe_df)

    futures_delivery_match_dce_df = futures_delivery_match_dce(symbol="a")
    print(futures_delivery_match_dce_df)

    futures_delivery_match_czce_df = futures_delivery_match_czce(date="20210106")
    print(futures_delivery_match_czce_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/6/4 18:30
Desc: 财新数据-指数报告-数字经济指数
https://yun.ccxe.com.cn/indices/dei
"""

import pandas as pd
import requests


def index_pmi_com_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-财新中国 PMI-综合 PMI
    https://yun.ccxe.com.cn/indices/pmi
    :return: 财新中国 PMI-综合 PMI
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "com"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "综合PMI", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "综合PMI",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_pmi_man_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-财新中国 PMI-制造业 PMI
    https://yun.ccxe.com.cn/indices/pmi
    :return: 财新中国 PMI-制造业 PMI
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "man"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "制造业PMI", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "制造业PMI",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_pmi_ser_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-财新中国 PMI-服务业 PMI
    https://yun.ccxe.com.cn/indices/pmi
    :return: 财新中国 PMI-服务业 PMI
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "ser"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "服务业PMI", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "服务业PMI",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_dei_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-数字经济指数
    https://yun.ccxe.com.cn/indices/dei
    :return: 数字经济指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "dei"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "数字经济指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "数字经济指数",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_ii_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-产业指数
    https://yun.ccxe.com.cn/indices/dei
    :return: 产业指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "ii"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "产业指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "产业指数",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_si_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-溢出指数
    https://yun.ccxe.com.cn/indices/dei
    :return: 溢出指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "si"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "溢出指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "溢出指数",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_fi_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-融合指数
    https://yun.ccxe.com.cn/indices/dei
    :return: 融合指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "fi"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "融合指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "融合指数",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_bi_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-基础指数
    https://yun.ccxe.com.cn/indices/dei
    :return: 基础指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "bi"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "基础指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "基础指数",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_nei_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-中国新经济指数
    https://yun.ccxe.com.cn/indices/nei
    :return: 中国新经济指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "nei"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "中国新经济指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "中国新经济指数",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_li_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-劳动力投入指数
    https://yun.ccxe.com.cn/indices/nei
    :return: 劳动力投入指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "li"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "劳动力投入指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "劳动力投入指数",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_ci_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-资本投入指数
    https://yun.ccxe.com.cn/indices/nei
    :return: 资本投入指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "ci"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "资本投入指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "资本投入指数",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_ti_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-科技投入指数
    https://yun.ccxe.com.cn/indices/nei
    :return: 科技投入指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "ti"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "科技投入指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "科技投入指数",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_neaw_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-新经济行业入职平均工资水平
    https://yun.ccxe.com.cn/indices/nei
    :return: 新经济行业入职平均工资水平
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "neaw"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "新经济行业入职平均工资水平", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "新经济行业入职平均工资水平",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_awpr_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-新经济入职工资溢价水平
    https://yun.ccxe.com.cn/indices/nei
    :return: 新经济入职工资溢价水平
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {"type": "awpr"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "新经济入职工资溢价水平", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "新经济入职工资溢价水平",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_cci_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-大宗商品指数
    https://yun.ccxe.com.cn/indices/nei
    :return: 大宗商品指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {
        "type": "cci",
        "code": "1000050",
        "month": "-1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化值", "大宗商品指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "大宗商品指数",
            "变化值",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_qli_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-高质量因子
    https://yun.ccxe.com.cn/indices/qli
    :return: 高质量因子
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {
        "type": "qli",
        "code": "1000050",
        "month": "-1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化幅度", "高质量因子指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "高质量因子指数",
            "变化幅度",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_ai_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-AI策略指数
    https://yun.ccxe.com.cn/indices/ai
    :return: AI策略指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {
        "type": "ai",
        "code": "1000050",
        "month": "-1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化幅度", "AI策略指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "AI策略指数",
            "变化幅度",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_bei_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-基石经济指数
    https://yun.ccxe.com.cn/indices/bei
    :return: 基石经济指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {
        "type": "ind",
        "code": "930927",
        "month": "-1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化幅度", "基石经济指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "基石经济指数",
            "变化幅度",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


def index_neei_cx() -> pd.DataFrame():
    """
    财新数据-指数报告-新动能指数
    https://yun.ccxe.com.cn/indices/neei
    :return: 新动能指数
    :rtype: pandas.DataFrame
    """
    url = "https://yun.ccxe.com.cn/api/index/pro/cxIndexTrendInfo"
    params = {
        "type": "ind",
        "code": "930928",
        "month": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["变化幅度", "新动能指数", "日期"]
    temp_df = temp_df[
        [
            "日期",
            "新动能指数",
            "变化幅度",
        ]
    ]
    temp_df["日期"] = (
        pd.to_datetime(temp_df["日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    return temp_df


if __name__ == "__main__":
    index_pmi_com_cx_df = index_pmi_com_cx()
    print(index_pmi_com_cx_df)

    index_pmi_man_cx_df = index_pmi_man_cx()
    print(index_pmi_man_cx_df)

    index_pmi_ser_cx_df = index_pmi_ser_cx()
    print(index_pmi_ser_cx_df)

    index_dei_cx_df = index_dei_cx()
    print(index_dei_cx_df)

    index_ii_cx_df = index_ii_cx()
    print(index_ii_cx_df)

    index_si_cx_df = index_si_cx()
    print(index_si_cx_df)

    index_fi_cx_df = index_fi_cx()
    print(index_fi_cx_df)

    index_bi_cx_df = index_bi_cx()
    print(index_bi_cx_df)

    index_nei_cx_df = index_nei_cx()
    print(index_nei_cx_df)

    index_li_cx_df = index_li_cx()
    print(index_li_cx_df)

    index_ci_cx_df = index_ci_cx()
    print(index_ci_cx_df)

    index_ti_cx_df = index_ti_cx()
    print(index_ti_cx_df)

    index_neaw_cx_df = index_neaw_cx()
    print(index_neaw_cx_df)

    index_awpr_cx_df = index_awpr_cx()
    print(index_awpr_cx_df)

    index_cci_cx_df = index_cci_cx()
    print(index_cci_cx_df)

    index_qli_cx_df = index_qli_cx()
    print(index_qli_cx_df)

    index_ai_cx_df = index_ai_cx()
    print(index_ai_cx_df)

    index_bei_cx_df = index_bei_cx()
    print(index_bei_cx_df)

    index_neei_cx_df = index_neei_cx()
    print(index_neei_cx_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/9/5 15:41
Desc: 芝加哥商业交易所-比特币成交量报告
https://datacenter.jin10.com/reportType/dc_cme_btc_report
"""

import pandas as pd
import requests


def crypto_bitcoin_cme(date: str = "20230830") -> pd.DataFrame:
    """
    芝加哥商业交易所-比特币成交量报告
    https://datacenter.jin10.com/reportType/dc_cme_btc_report
    :return: 比特币成交量报告
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-api.jin10.com/reports/list"
    params = {
        "category": "cme",
        "date": "-".join([date[:4], date[4:6], date[6:]]),
        "attr_id": "4",
    }
    headers = {
        "accept": "*/*",
        "accept-encoding": "gzip, deflate, br",
        "accept-language": "zh-CN,zh;q=0.9,en;q=0.8",
        "cache-control": "no-cache",
        "origin": "https://datacenter.jin10.com",
        "pragma": "no-cache",
        "referer": "https://datacenter.jin10.com/",
        "sec-ch-ua": '" Not;A Brand";v="99", "Google Chrome";v="91", "Chromium";v="91"',
        "sec-ch-ua-mobile": "?0",
        "sec-fetch-dest": "empty",
        "sec-fetch-mode": "cors",
        "sec-fetch-site": "same-site",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.106 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "",
        "x-version": "1.0.0",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(
        [item for item in data_json["data"]["values"]],
        columns=[item["name"] for item in data_json["data"]["keys"]],
    )
    temp_df["电子交易合约"] = pd.to_numeric(temp_df["电子交易合约"], errors="coerce")
    temp_df["场内成交合约"] = pd.to_numeric(temp_df["场内成交合约"], errors="coerce")
    temp_df["场外成交合约"] = pd.to_numeric(temp_df["场外成交合约"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["未平仓合约"] = pd.to_numeric(temp_df["未平仓合约"], errors="coerce")
    temp_df["持仓变化"] = pd.to_numeric(temp_df["持仓变化"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    crypto_bitcoin_cme_df = crypto_bitcoin_cme(date="20230830")
    print(crypto_bitcoin_cme_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/17 19:30
Desc: 东方财富网-期货行情-品种及交易规则
https://portal.eastmoneyfutures.com/pages/service/jyts.html#jyrl
"""

import pandas as pd
import requests
from akshare.utils.cons import headers


def futures_rule_em() -> pd.DataFrame:
    """
    东方财富网-期货行情-品种及交易规则
    https://portal.eastmoneyfutures.com/pages/service/jyts.html#jyrl
    :return: 品种及交易规则
    :rtype: pandas.DataFrame
    """
    url = "https://eastmoneyfutures.com/api/ComManage/GetPZJYInfo"
    r = requests.get(url, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["Data"])
    return temp_df


def futures_trading_hours_em():
    """
    东方财富网-期货交易时间
    https://qhweb.eastmoney.com/tradinghours
    """
    pass


if __name__ == "__main__":
    futures_rule_em_df = futures_rule_em()
    print(futures_rule_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/12 17:00
Desc: 东方财富-沪深板块-概念板块
https://quote.eastmoney.com/center/boardlist.html#concept_board
"""

import re
from functools import lru_cache

import pandas as pd
import requests

from akshare.utils.func import fetch_paginated_data


@lru_cache()
def __stock_board_concept_name_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深京板块-概念板块-名称
    https://quote.eastmoney.com/center/boardlist.html#concept_board
    :return: 概念板块-名称
    :rtype: pandas.DataFrame
    """
    url = "https://79.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f12",
        "fs": "m:90 t:3 f:!50",
        "fields": "f2,f3,f4,f8,f12,f14,f15,f16,f17,f18,f20,f21,f24,f25,f22,f33,f11,f62,f128,f124,f107,f104,f105,f136",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.columns = [
        "排名",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "换手率",
        "_",
        "板块代码",
        "板块名称",
        "_",
        "_",
        "_",
        "_",
        "总市值",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "上涨家数",
        "下跌家数",
        "_",
        "_",
        "领涨股票",
        "_",
        "_",
        "领涨股票-涨跌幅",
    ]
    temp_df = temp_df[
        [
            "排名",
            "板块名称",
            "板块代码",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "总市值",
            "换手率",
            "上涨家数",
            "下跌家数",
            "领涨股票",
            "领涨股票-涨跌幅",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    temp_df["上涨家数"] = pd.to_numeric(temp_df["上涨家数"], errors="coerce")
    temp_df["下跌家数"] = pd.to_numeric(temp_df["下跌家数"], errors="coerce")
    temp_df["领涨股票-涨跌幅"] = pd.to_numeric(
        temp_df["领涨股票-涨跌幅"], errors="coerce"
    )
    return temp_df


def stock_board_concept_name_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深京板块-概念板块-名称
    https://quote.eastmoney.com/center/boardlist.html#concept_board
    :return: 概念板块-名称
    :rtype: pandas.DataFrame
    """
    url = "https://79.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f12",
        "fs": "m:90 t:3 f:!50",
        "fields": "f2,f3,f4,f8,f12,f14,f15,f16,f17,f18,f20,f21,f24,f25,f22,f33,f11,f62,f128,f124,f107,f104,f105,f136",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.columns = [
        "排名",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "换手率",
        "_",
        "板块代码",
        "板块名称",
        "_",
        "_",
        "_",
        "_",
        "总市值",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "上涨家数",
        "下跌家数",
        "_",
        "_",
        "领涨股票",
        "_",
        "_",
        "领涨股票-涨跌幅",
    ]
    temp_df = temp_df[
        [
            "排名",
            "板块名称",
            "板块代码",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "总市值",
            "换手率",
            "上涨家数",
            "下跌家数",
            "领涨股票",
            "领涨股票-涨跌幅",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    temp_df["上涨家数"] = pd.to_numeric(temp_df["上涨家数"], errors="coerce")
    temp_df["下跌家数"] = pd.to_numeric(temp_df["下跌家数"], errors="coerce")
    temp_df["领涨股票-涨跌幅"] = pd.to_numeric(
        temp_df["领涨股票-涨跌幅"], errors="coerce"
    )
    return temp_df


def stock_board_concept_spot_em(symbol: str = "可燃冰") -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深京板块-概念板块-实时行情
    https://quote.eastmoney.com/bk/90.BK0818.html
    :return: 概念板块-实时行情
    :rtype: pandas.DataFrame
    """
    url = "https://91.push2.eastmoney.com/api/qt/stock/get"
    field_map = {
        "f43": "最新",
        "f44": "最高",
        "f45": "最低",
        "f46": "开盘",
        "f47": "成交量",
        "f48": "成交额",
        "f170": "涨跌幅",
        "f171": "振幅",
        "f168": "换手率",
        "f169": "涨跌额",
    }

    if re.match(pattern=r"^BK\d+", string=symbol):
        em_code = symbol
    else:
        industry_listing = __stock_board_concept_name_em()
        em_code = industry_listing.query("板块名称 == @symbol")["板块代码"].values[0]
    params = dict(
        fields=",".join(field_map.keys()),
        mpi="1000",
        invt="2",
        fltt="1",
        secid=f"90.{em_code}",
    )
    r = requests.get(url, params=params)
    data_dict = r.json()
    result = pd.DataFrame.from_dict(data_dict["data"], orient="index")
    result.rename(field_map, inplace=True)
    result.reset_index(inplace=True)
    result.columns = ["item", "value"]
    result["value"] = pd.to_numeric(result["value"], errors="coerce")

    # 各项转换成正常单位. 除了成交量与成交额, 原始数据中已是正常单位(元)
    result["value"] = result["value"] * 1e-2
    result.iloc[4, 1] = result.iloc[4, 1] * 1e2
    result.iloc[5, 1] = result.iloc[5, 1] * 1e2
    return result


def stock_board_concept_hist_em(
    symbol: str = "绿色电力",
    period: str = "daily",
    start_date: str = "20220101",
    end_date: str = "20221128",
    adjust: str = "",
) -> pd.DataFrame:
    """
    东方财富网-沪深板块-概念板块-历史行情
    https://quote.eastmoney.com/bk/90.BK0715.html
    :param symbol: 板块名称
    :type symbol: str
    :type period: 周期; choice of {"daily", "weekly", "monthly"}
    :param period: 板块名称
    :param start_date: 开始时间
    :type start_date: str
    :param end_date: 结束时间
    :type end_date: str
    :param adjust: choice of {'': 不复权, "qfq": 前复权, "hfq": 后复权}
    :type adjust: str
    :return: 历史行情
    :rtype: pandas.DataFrame
    """
    period_map = {
        "daily": "101",
        "weekly": "102",
        "monthly": "103",
    }
    stock_board_concept_em_map = __stock_board_concept_name_em()
    stock_board_code = stock_board_concept_em_map[
        stock_board_concept_em_map["板块名称"] == symbol
    ]["板块代码"].values[0]
    adjust_map = {"": "0", "qfq": "1", "hfq": "2"}
    url = "https://91.push2his.eastmoney.com/api/qt/stock/kline/get"
    params = {
        "secid": f"90.{stock_board_code}",
        "fields1": "f1,f2,f3,f4,f5,f6",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
        "klt": period_map[period],
        "fqt": adjust_map[adjust],
        "beg": start_date,
        "end": end_date,
        "smplmt": "10000",
        "lmt": "1000000",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    temp_df.columns = [
        "日期",
        "开盘",
        "收盘",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "振幅",
        "涨跌幅",
        "涨跌额",
        "换手率",
    ]
    temp_df = temp_df[
        [
            "日期",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "涨跌幅",
            "涨跌额",
            "成交量",
            "成交额",
            "振幅",
            "换手率",
        ]
    ]
    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    return temp_df


def stock_board_concept_hist_min_em(
    symbol: str = "长寿药", period: str = "5"
) -> pd.DataFrame:
    """
    东方财富网-沪深板块-概念板块-分时历史行情
    https://quote.eastmoney.com/bk/90.BK0715.html
    :param symbol: 板块名称
    :type symbol: str
    :param period: choice of {"1", "5", "15", "30", "60"}
    :type period: str
    :return: 分时历史行情
    :rtype: pandas.DataFrame
    """
    stock_board_concept_em_map = __stock_board_concept_name_em()
    stock_board_code = stock_board_concept_em_map[
        stock_board_concept_em_map["板块名称"] == symbol
    ]["板块代码"].values[0]
    if period == "1":
        url = "https://push2his.eastmoney.com/api/qt/stock/trends2/get"
        params = {
            "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
            "iscr": "0",
            "ndays": "1",
            "secid": f"90.{stock_board_code}",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            [item.split(",") for item in data_json["data"]["trends"]]
        )
        temp_df.columns = [
            "日期时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "最新价",
        ]
        temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
        temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
        temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
        temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
        temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
        return temp_df
    else:
        url = "https://91.push2his.eastmoney.com/api/qt/stock/kline/get"
        params = {
            "secid": f"90.{stock_board_code}",
            "fields1": "f1,f2,f3,f4,f5,f6",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
            "klt": period,
            "fqt": "1",
            "end": "20500101",
            "lmt": "1000000",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            [item.split(",") for item in data_json["data"]["klines"]]
        )
        temp_df.columns = [
            "日期时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "振幅",
            "涨跌幅",
            "涨跌额",
            "换手率",
        ]
        temp_df = temp_df[
            [
                "日期时间",
                "开盘",
                "收盘",
                "最高",
                "最低",
                "涨跌幅",
                "涨跌额",
                "成交量",
                "成交额",
                "振幅",
                "换手率",
            ]
        ]
        temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
        temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
        temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
        temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
        temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
        temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
        temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
        temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
        return temp_df


def stock_board_concept_cons_em(symbol: str = "融资融券") -> pd.DataFrame:
    """
    东方财富-沪深板块-概念板块-板块成份
    https://quote.eastmoney.com/center/boardlist.html#boards-BK06551
    :param symbol: 板块名称或者板块代码
    :type symbol: str
    :return: 板块成份
    :rtype: pandas.DataFrame
    """
    if re.match(pattern=r"^BK\d+", string=symbol):
        stock_board_code = symbol
    else:
        stock_board_concept_em_map = __stock_board_concept_name_em()
        stock_board_code = stock_board_concept_em_map[
            stock_board_concept_em_map["板块名称"] == symbol
        ]["板块代码"].values[0]
    url = "https://29.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f12",
        "fs": f"b:{stock_board_code} f:!50",
        "fields": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,"
        "f24,f25,f22,f11,f62,f128,f136,f115,f152,f45",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.columns = [
        "序号",
        "_",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "成交量",
        "成交额",
        "振幅",
        "换手率",
        "市盈率-动态",
        "_",
        "_",
        "代码",
        "_",
        "名称",
        "最高",
        "最低",
        "今开",
        "昨收",
        "_",
        "_",
        "_",
        "市净率",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "涨跌额",
            "成交量",
            "成交额",
            "振幅",
            "最高",
            "最低",
            "今开",
            "昨收",
            "换手率",
            "市盈率-动态",
            "市净率",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    temp_df["市盈率-动态"] = pd.to_numeric(temp_df["市盈率-动态"], errors="coerce")
    temp_df["市净率"] = pd.to_numeric(temp_df["市净率"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_board_concept_em_df = stock_board_concept_name_em()
    print(stock_board_concept_em_df)

    stock_board_concept_spot_em_df = stock_board_concept_spot_em(symbol="可燃冰")
    print(stock_board_concept_spot_em_df)

    stock_board_concept_hist_em_df = stock_board_concept_hist_em(
        symbol="绿色电力",
        period="daily",
        start_date="20220101",
        end_date="20250227",
        adjust="",
    )
    print(stock_board_concept_hist_em_df)

    stock_board_concept_hist_min_em_df = stock_board_concept_hist_min_em(
        symbol="长寿药", period="5"
    )
    print(stock_board_concept_hist_min_em_df)

    stock_board_concept_cons_em_df = stock_board_concept_cons_em(symbol="BK0655")
    print(stock_board_concept_cons_em_df)
